/*
    Diy PS3 controller
    Copyright (C) 2010  Mathieu Laurendeau

    Original code from the Pan-Galactic Gargantuan Gargle Brain
    Copyright (C) 2009  Richard Burke

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "usb_joystick.h"
#include "RingBuffer.h"
#include "KeyboardDriver.h"
#include "MouseDriver.h"

#include "led.h"

/*
 ********************************************************************************
 * Local Functions and Variables
 ********************************************************************************
 */
#define CPU_PRESCALE(n) (CLKPR = 0x80, CLKPR = (n))

// simple function to press X button every 11 seconds
void tapX( void )
{
	usb_joystick_press(CROSS_BUTTON);
	usb_joystick_send();
	_delay_ms(100);
	usb_joystick_release(CROSS_BUTTON);
	usb_joystick_send();

	_delay_ms(100);

	usb_joystick_press(CROSS_BUTTON);
	usb_joystick_send();
	_delay_ms(100);
	usb_joystick_release(CROSS_BUTTON);
	usb_joystick_send();

	for (int k=0; k<=11; k++)
	_delay_ms(1000);
}

/*
 ********************************************************************************
 * main
 ********************************************************************************
 */
int main( void )
{
    /* Disable watchdog if enabled by bootloader/fuses */
    MCUSR &= ~(1 << WDRF);
    wdt_disable();

    /* Disable clock division */
    CPU_PRESCALE(0);
    LED_CONFIG;
    LED_ON;
    
    /* Initialize USB Joystick variables */
    usb_joystick_raz();

    /* Enable interrupts */
    sei( );

    /* Initialize USB Joystick */
    usb_init();
    while (!usb_configured()) ;
    _delay_ms(1000);
    LED_OFF;

    /* Send a first packet */
    usb_joystick_send();

    while( 1 )
    {
		// ************************************************************
    	// Implement your macros here....
		// ************************************************************
    	usb_joystick_press(CROSS_BUTTON);
    	usb_joystick_send();
    	_delay_ms(100);
    	usb_joystick_release(CROSS_BUTTON);
    	usb_joystick_send();

    	// 10 Seconds after x gets hit the race starts
    	// Race lasts 9 mins when no buttons are pressed
    	// Takes 40 seconds to get to money
		// NOTE: _delay_ms will only work for 6 seconds therefore we need to
		//            use a for loop.
    	for (int i=0; i <= 551; i++)
    		_delay_ms(1000);		// Race+Money takes 9.5 minutes

    	usb_joystick_press(CROSS_BUTTON);
    	usb_joystick_send();
    	_delay_ms(100);
    	usb_joystick_release(CROSS_BUTTON);
    	usb_joystick_send();

    	_delay_ms(4000);		// Wait for xp

    	usb_joystick_press(CROSS_BUTTON);
    	usb_joystick_send();
    	_delay_ms(100);
    	usb_joystick_release(CROSS_BUTTON);
    	usb_joystick_send();

    	_delay_ms(4000);		// Wait for driver

    	usb_joystick_press(CIRCLE_BUTTON);
    	usb_joystick_send();
    	_delay_ms(100);
    	usb_joystick_release(CIRCLE_BUTTON);
    	usb_joystick_send();
    	_delay_ms(1200);
    	usb_joystick_press(CROSS_BUTTON);
    	usb_joystick_send();
    	_delay_ms(100);
    	usb_joystick_release(CROSS_BUTTON);
    	usb_joystick_send();

    	_delay_ms(3000);
    }

    return 0;
}

